@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  *
- $Id: power-management.c,v 1.11 2005/08/30 14:32:42 rodrigo Exp $
+ $Id: power-management.c,v 1.13 2005/11/23 16:15:05 gpastore Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -171,7 +171,7 @@ apm_readinfo (BatteryStatus *status)
 #undef __linux__
 #undef HAVE_HAL
 
-#elif __FreeBSD__
+#elif defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
 
 #include <machine/apm_bios.h>
 #include "acpi-freebsd.h"
@@ -277,7 +277,7 @@ apm_readinfo (BatteryStatus *status)
 #include <apm.h>
 #include "acpi-linux.h"
 
-struct acpi_info acpiinfo;
+static struct acpi_info acpiinfo;
 static gboolean using_acpi;
 static int acpi_count;
 static int acpiwatch;
@@ -473,7 +473,7 @@ power_management_initialise( int no_hal )
         G_IO_IN | G_IO_ERR | G_IO_HUP,
         acpi_callback, NULL);
   }
-#elif defined(__FreeBSD__)
+#elif defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
   if (acpi_freebsd_init(&acpiinfo)) {
     using_acpi = TRUE;
     acpi_count = 0;
@@ -511,7 +511,7 @@ power_management_cleanup( void )
      acpiwatch = 0;
      acpi_linux_cleanup(&acpiinfo);
   }
-#elif defined(__FreeBSD__)
+#elif defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
   if (using_acpi) {
     acpi_freebsd_cleanup(&acpiinfo);
   }
