@@ -22,6 +22,7 @@
 #include <X11/Xatom.h>
 #include <gdk/gdkx.h>
 #define WNCK_I_KNOW_THIS_IS_UNSTABLE 1
+#include <libgnome/libgnome.h>
 #include <libwnck/libwnck.h>
 #include <string.h>
 
@@ -30,6 +31,9 @@
 #include "util.h"
 #include "stickynotes_applet.h"
 
+/* Stop gcc complaining about xmlChar's signedness */
+#define XML_CHAR(str) ((xmlChar *) (str))
+
 static void response_cb (GtkWidget *dialog, gint id, gpointer data);
 
 /* Based on a function found in wnck */
@@ -57,9 +61,10 @@ set_icon_geometry  (GdkWindow *window,
                        (guchar *)&data, 4);
 }
 
-/* Create a new (empty) Sticky Note */
+/* Create a new (empty) Sticky Note at a specific position 
+   and with specific size */
 StickyNote *
-stickynote_new (GdkScreen *screen)
+stickynote_new_aux (GdkScreen *screen, gint x, gint y, gint w, gint h)
 {
 	StickyNote *note;
 	GladeXML *window;
@@ -78,6 +83,9 @@ stickynote_new (GdkScreen *screen)
 
 	note->w_window = glade_xml_get_widget (window, "stickynote_window");
 	gtk_window_set_screen(GTK_WINDOW(note->w_window),screen);
+	gtk_window_set_decorated (GTK_WINDOW (note->w_window), FALSE);
+	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (note->w_window), TRUE);
+	gtk_window_set_skip_pager_hint (GTK_WINDOW (note->w_window), TRUE);
 
 	note->w_title = glade_xml_get_widget(window, "title_label");
 	note->w_body = glade_xml_get_widget(window, "body_text");
@@ -126,20 +134,31 @@ stickynote_new (GdkScreen *screen)
 	note->font_color = NULL;
 	note->font = NULL;
 	note->locked = FALSE;
-	note->x = -1;
-	note->y = -1;
-	note->w = 0;
-	note->h = 0;
+	note->x = x;
+	note->y = y;
+	note->w = w;
+	note->h = h;
 
 	/* Customize the window */
 	if (gconf_client_get_bool(stickynotes->gconf,
 				GCONF_PATH "/settings/sticky", NULL))
 		gtk_window_stick(GTK_WINDOW(note->w_window));
-	gtk_window_resize(GTK_WINDOW(note->w_window),
-			gconf_client_get_int(stickynotes->gconf,
-				GCONF_PATH "/defaults/width", NULL),
-			gconf_client_get_int(stickynotes->gconf,
-				GCONF_PATH "/defaults/height", NULL));
+
+	if (w == 0 || h == 0)
+		gtk_window_resize (GTK_WINDOW(note->w_window),
+				gconf_client_get_int(stickynotes->gconf,
+					GCONF_PATH "/defaults/width", NULL),
+				gconf_client_get_int(stickynotes->gconf,
+					GCONF_PATH "/defaults/height", NULL));
+	else
+		gtk_window_resize (GTK_WINDOW(note->w_window),
+				note->w,
+				note->h);
+
+	if (x != -1 && y != -1)
+		gtk_window_move (GTK_WINDOW(note->w_window),
+				note->x,
+				note->y);
 
 	/* Set the button images */
 	size = gtk_icon_size_from_name ("stickynotes_icon_size");
@@ -161,6 +180,7 @@ stickynote_new (GdkScreen *screen)
 
 	gtk_widget_realize (note->w_window);
 
+	/* gnome_popup_menu_*() are deprecated. Rewrite to use gtk_menu_popup_*()? */
 	/* Connect a popup menu to all buttons and title */
 	gnome_popup_menu_attach(note->w_menu, note->w_window, note);
 	gnome_popup_menu_attach(note->w_menu, note->w_lock, note);
@@ -222,6 +242,8 @@ stickynote_new (GdkScreen *screen)
 			G_CALLBACK (properties_apply_font_cb), note);
 	g_signal_connect (G_OBJECT (note->w_entry), "activate",
 			G_CALLBACK (properties_activate_cb), note);
+	g_signal_connect (G_OBJECT (note->w_properties), "delete-event",
+			G_CALLBACK (gtk_widget_hide), note);
 
 	/* unref GladeXML objects */
 	g_object_unref(properties);
@@ -231,6 +253,13 @@ stickynote_new (GdkScreen *screen)
 	return note;
 }
 
+/* Create a new (empty) Sticky Note */
+StickyNote *
+stickynote_new (GdkScreen *screen)
+{
+	return stickynote_new_aux (screen, -1, -1, 0, 0);
+}
+
 /* Destroy a Sticky Note */
 void stickynote_free(StickyNote *note)
 {
@@ -291,15 +320,17 @@ void stickynote_set_title(StickyNote *note, const gchar *title)
 {
 	/* If title is NULL, use the current date as the title. */
 	if (!title) {
-		gchar *date_title;
+		gchar *date_title, *tmp;
 		gchar *date_format = gconf_client_get_string(stickynotes->gconf, GCONF_PATH "/settings/date_format", NULL);
 		if (!date_format)
 			date_format = g_strdup ("%x");
-		date_title = get_current_date(date_format);
+		tmp = get_current_date (date_format);
+		date_title = g_locale_to_utf8 (tmp, -1, NULL, NULL, NULL);
 
 		gtk_window_set_title(GTK_WINDOW(note->w_window), date_title);
 		gtk_label_set_text(GTK_LABEL (note->w_title), date_title);
 
+		g_free (tmp);
 		g_free(date_title);
 		g_free(date_format);
 	}
@@ -544,11 +575,6 @@ stickynote_set_visible (StickyNote *note, gboolean visible)
 {
 	if (visible)
 	{
-		gtk_window_set_decorated (GTK_WINDOW (note->w_window), FALSE);
-		gtk_window_set_skip_taskbar_hint (GTK_WINDOW (note->w_window),
-				TRUE);
-		gtk_window_set_skip_pager_hint (GTK_WINDOW (note->w_window),
-				TRUE);
 		gtk_window_present (GTK_WINDOW (note->w_window));
 		
 		if (note->x != -1 || note->y != -1)
@@ -654,11 +680,11 @@ stickynotes_save (void)
 	gint i;
 	
 	/* Create a new XML document */
-	xmlDocPtr doc = xmlNewDoc("1.0");
-	xmlNodePtr root = xmlNewDocNode(doc, NULL, "stickynotes", NULL);
+	xmlDocPtr doc = xmlNewDoc(XML_CHAR ("1.0"));
+	xmlNodePtr root = xmlNewDocNode(doc, NULL, XML_CHAR ("stickynotes"), NULL);
 
 	xmlDocSetRootElement(doc, root);
-	xmlNewProp(root, "version", VERSION);
+	xmlNewProp(root, XML_CHAR("version"), XML_CHAR (VERSION));
 
 	wnck_screen = wnck_screen_get_default ();
 	wnck_screen_force_update (wnck_screen);
@@ -702,29 +728,29 @@ stickynotes_save (void)
 
 		/* Save the note as a node in the XML document */
 		{
-			xmlNodePtr node = xmlNewTextChild(root, NULL, "note",
-					body);		
-			xmlNewProp(node, "title", title);
+			xmlNodePtr node = xmlNewTextChild(root, NULL, XML_CHAR ("note"),
+					XML_CHAR (body));		
+			xmlNewProp(node, XML_CHAR ("title"), XML_CHAR (title));
 			if (note->color)
-				xmlNewProp (node, "color", note->color);
+				xmlNewProp (node, XML_CHAR ("color"), XML_CHAR (note->color));
 			if (note->font_color)
-				xmlNewProp (node, "font_color",
-						note->font_color);
+				xmlNewProp (node, XML_CHAR ("font_color"),
+						XML_CHAR (note->font_color));
 			if (note->font)
-				xmlNewProp (node, "font", note->font);
+				xmlNewProp (node, XML_CHAR ("font"), XML_CHAR (note->font));
 			if (note->locked)
-				xmlNewProp (node, "locked", "true");
-			xmlNewProp (node, "x", x_str);
-			xmlNewProp (node, "y", y_str);
-			xmlNewProp (node, "w", w_str);
-			xmlNewProp (node, "h", h_str);
+				xmlNewProp (node, XML_CHAR ("locked"), XML_CHAR ("true"));
+			xmlNewProp (node, XML_CHAR ("x"), XML_CHAR (x_str));
+			xmlNewProp (node, XML_CHAR ("y"), XML_CHAR (y_str));
+			xmlNewProp (node, XML_CHAR ("w"), XML_CHAR (w_str));
+			xmlNewProp (node, XML_CHAR ("h"), XML_CHAR (h_str));
 			if (note->workspace > 0)
 			{
 				char *workspace_str;
 				
 				workspace_str = g_strdup_printf ("%i",
 						note->workspace);
-				xmlNewProp (node, "workspace", workspace_str);
+				xmlNewProp (node, XML_CHAR ("workspace"), XML_CHAR (workspace_str));
 				g_free (workspace_str);
 			}
 		}
@@ -757,10 +783,10 @@ stickynotes_load (GdkScreen *screen)
 	xmlDocPtr doc;
 	xmlNodePtr root;
 	xmlNodePtr node;
-	WnckScreen *wnck_screen;
-	GList *new_notes, *tmp1;  // Lists of StickyNote*'s
-	GList *new_nodes, *tmp2;  // Lists of xmlNodePtr's
-
+	/* WnckScreen *wnck_screen; */
+	GList *new_notes, *tmp1;  /* Lists of StickyNote*'s */
+	GList *new_nodes; /* Lists of xmlNodePtr's */
+	int x, y, w, h;
 	/* The XML file is $HOME/.gnome2/stickynotes_applet, most probably */
 	{
 		gchar *file = g_strdup_printf("%s%s", g_get_home_dir(),
@@ -778,7 +804,7 @@ stickynotes_load (GdkScreen *screen)
 	
 	/* If the XML file is corrupted/incorrect, create a blank one */
 	root = xmlDocGetRootElement(doc);
-	if (!root || xmlStrcmp(root->name, (const xmlChar *) "stickynotes"))
+	if (!root || xmlStrcmp(root->name, XML_CHAR ("stickynotes")))
 	{
 		xmlFreeDoc(doc);
 		stickynotes_save();
@@ -793,8 +819,47 @@ stickynotes_load (GdkScreen *screen)
 	while (node) {
 		if (!xmlStrcmp(node->name, (const xmlChar *) "note"))
 		{
+			/* Retrieve and set the window size of the note */
+			{
+				gchar *w_str = (gchar *)xmlGetProp (node, XML_CHAR ("w"));
+				gchar *h_str = (gchar *)xmlGetProp (node, XML_CHAR ("h"));
+				if (w_str && h_str)
+				{
+					w = atoi (w_str);
+					h = atoi (h_str);
+				}
+				else
+				{
+					w = 0;
+					h = 0;
+				}
+
+				g_free (w_str);
+				g_free (h_str);
+			}
+			
+			/* Retrieve and set the window position of the note */
+			{
+				gchar *x_str = (gchar *)xmlGetProp (node, XML_CHAR ("x"));
+				gchar *y_str = (gchar *)xmlGetProp (node, XML_CHAR ("y"));
+
+				if (x_str && y_str)
+				{
+					x = atoi (x_str);
+					y = atoi (y_str);
+				}
+				else
+				{
+					x = -1;
+					y = -1;
+				}
+
+				g_free (x_str);
+				g_free (y_str);
+			}
+
 			/* Create a new note */
-			StickyNote *note = stickynote_new (screen);
+			StickyNote *note = stickynote_new_aux (screen, x, y, w, h);
 			stickynotes->notes = g_list_append (stickynotes->notes,
 					note);
 			new_notes = g_list_append (new_notes, note);
@@ -802,7 +867,7 @@ stickynotes_load (GdkScreen *screen)
 
 			/* Retrieve and set title of the note */
 			{
-				gchar *title = xmlGetProp(node, "title");
+				gchar *title = (gchar *)xmlGetProp(node, XML_CHAR ("title"));
 				if (title)
 					stickynote_set_title (note, title);
 				g_free (title);
@@ -810,12 +875,11 @@ stickynotes_load (GdkScreen *screen)
 
 			/* Retrieve and set the color of the note */
 			{
-				char *color_str;
-				char *font_color_str;
+				gchar *color_str;
+				gchar *font_color_str;
 				
-				color_str = xmlGetProp (node, "color");
-				font_color_str = xmlGetProp (node,
-						"font_color");
+				color_str = (gchar *)xmlGetProp (node, XML_CHAR ("color"));
+				font_color_str = (gchar *)xmlGetProp (node, XML_CHAR ("font_color"));
 				
 				if (color_str || font_color_str)
 					stickynote_set_color (note,
@@ -828,56 +892,18 @@ stickynotes_load (GdkScreen *screen)
 
 			/* Retrieve and set the font of the note */
 			{
-				gchar *font_str = xmlGetProp(node, "font");
+				gchar *font_str = (gchar *)xmlGetProp(node, XML_CHAR ("font"));
 				if (font_str)
 					stickynote_set_font (note, font_str,
 							TRUE);
 				g_free(font_str);
 			}
 
-			/* Retrieve and set the window size of the note */
-			{
-				gchar *w_str = xmlGetProp(node, "w");
-				gchar *h_str = xmlGetProp(node, "h");
-				if (w_str && h_str)
-					gtk_window_resize (GTK_WINDOW (
-							note->w_window),
-							atoi(w_str),
-							atoi(h_str));
-				gtk_window_get_size (GTK_WINDOW (
-							note->w_window),
-						&note->w, &note->h);
-				g_free(w_str);
-				g_free(h_str);
-			}
-			
-			/* Retrieve and set the window position of the note */
-			{
-				gchar *x_str = xmlGetProp(node, "x");
-				gchar *y_str = xmlGetProp(node, "y");
-				if (x_str && y_str)
-				{
-					if (atoi(x_str) != -1 ||
-							atoi(y_str) !=-1)
-					{
-					    gtk_window_move (GTK_WINDOW (
-							note->w_window),
-							    atoi(x_str),
-							    atoi(y_str));
-					}
-					gtk_window_get_position (GTK_WINDOW (
-							note->w_window),
-							&note->x, &note->y);
-					g_free(x_str);
-					g_free(y_str);
-				}
-			}
-			
 			/* Retrieve the workspace */
 			{
 				char *workspace_str;
 
-				workspace_str = xmlGetProp (node, "workspace");
+				workspace_str = (gchar *)xmlGetProp (node, XML_CHAR ("workspace"));
 				if (workspace_str)
 				{
 					note->workspace = atoi (workspace_str);
@@ -888,7 +914,7 @@ stickynotes_load (GdkScreen *screen)
 			/* Retrieve and set (if any) the body contents of the
 			 * note */
 			{
-				gchar *body = xmlNodeListGetString(doc,
+				gchar *body = (gchar *)xmlNodeListGetString(doc,
 						node->xmlChildrenNode, 1);
 				if (body) {
 					GtkTextBuffer *buffer;
@@ -907,7 +933,7 @@ stickynotes_load (GdkScreen *screen)
 			/* Retrieve and set the locked state of the note,
 			 * by default unlocked */
 			{
-				gchar *locked = xmlGetProp(node, "locked");
+				gchar *locked = (gchar *)xmlGetProp(node, XML_CHAR ("locked"));
 				if (locked)
 					stickynote_set_locked(note,
 						!strcmp(locked, "true"));
